#!/usr/bin/env node

// src/index.ts
import { program } from "commander";
import chalk2 from "chalk";
import prettier from "prettier";
import fs from "fs";

// src/types.ts
import chalk from "chalk";
var propertyTypes = {
  Float: "number",
  Int: "number",
  String: "string",
  Text: "string",
  Vector2: "Vector2",
  File: "string",
  Color: "Color",
  Tile: "Tile"
};
var getPropertyType = (key) => {
  const v = propertyTypes[key];
  if (v)
    return v;
  if (key.startsWith("Enum:")) {
    return key.slice(5);
  }
  console.warn(chalk.yellow(`Unknown type: ${key}`));
  return "unknown";
};

// src/index.ts
import path from "path";
await program.parseAsync();
var url = program.args[0];
if (!url) {
  console.error(chalk2.red("Usage: adofai-typegen <url>"));
  process.exit(1);
}
console.log(chalk2.blue`Fetching properties json`);
var data = await (await fetch(url)).json();
console.log(chalk2.blue`Generating...`);
var result = [];
result.push(`

export interface Action<T extends LevelEventType> {
  floor: number
  type: T
}

export type Tile = [number, TileType]

export enum TileType {
  ThisTile = "ThisTile",
  FirstTile = "FirstTile",
  LastTile = "LastTile"
}

export type Color = \`#\${string}\`

export type Vector2 = [number, number]

`);
var enumValues = [];
var enumImports = /* @__PURE__ */ new Set();
var blacklistedKeys = ["type", "floor"];
for (const event of data.levelEvents) {
  console.log(chalk2.gray(`> ${event.name}`));
  const properties = [];
  for (const property of event.properties) {
    if (blacklistedKeys.includes(property.name)) {
      continue;
    }
    if (property.type.startsWith("Enum:"))
      enumImports.add(property.type.slice(5));
    properties.push(`${property.name}: ${getPropertyType(property.type)}`);
  }
  enumValues.push(event.name);
  result.push(`export interface ${event.name} extends Action<LevelEventType.${event.name}> {
    ${properties.join("\n")}
  }`);
}
result.unshift(`
import { ${Array.from(enumImports).join(", ")} } from './types'

export enum LevelEventType {
  ${enumValues.map((x) => `${x} = ${JSON.stringify(x)}`).join(",\n")}
}
`);
console.log(chalk2.blue`Saving...`);
var formatted = prettier.format(result.join("\n\n"), { parser: "typescript" });
await fs.promises.writeFile("output.ts", formatted);
console.log(
  chalk2.green`Done! You should implement below enums at ${path.join(
    process.cwd(),
    "types.ts"
  )}`
);
for (const i of enumImports) {
  console.log(chalk2.yellow(i));
}
